/*************************************************************
*
* Description:
*	Example program  for pC/OS & pC/SFS V1.14b
*
* Author[s]:
*	Frank Goetze
**************************************************************/

#include "../pC_SFS/inc/pC_SFS.h"
#include "../pC_SFS/inc/SFS_string.h"

#define OS_SYSTEM_TICKS_DIV   (OS_TIMER_TICKS_PER_SEC / OS_SYSTEM_TICKS_PER_SEC)

#define TASK_STK_SIZE         1024

/*************************************************************/

#ifdef _OS_GD32VF103C_
 #define pCOS_GD32VF103C_START    0
 #define pCOS_GD32VF103C_LONGAN   1

 #define GD32VF103C_TYPE       pCOS_GD32VF103C_LONGAN

 #if (GD32VF103C_TYPE == pCOS_GD32VF103C_START)     // -- START board
  #define LED_PORT     GPIOA
  #define LED_PIN      GPIO_PIN_7
 #endif
 #if (GD32VF103C_TYPE == pCOS_GD32VF103C_LONGAN)    // -- Sipeed Longan Nano
  #define LED_PORT     GPIOA
  #define LED_PIN      GPIO_PIN_1
 #endif

 static inline uint32_t SysTick_Config(uint32_t ticks)
 {
    eclic_global_interrupt_enable();
    *(uint64_t*)(TIMER_CTRL_ADDR + TIMER_MTIMECMP) = ticks;
    *(uint64_t*)(TIMER_CTRL_ADDR + TIMER_MTIME) = 0;
    eclic_set_nlbits(ECLIC_GROUP_LEVEL2_PRIO2);
    eclic_irq_enable(CLIC_INT_TMR, 3, 3);        // interrupt nr.7 - eclic_mtip_handler
    return (0UL);
 }

 void SysTick_init(void)
 {
    SystemCoreClockUpdate();
    if (SysTick_Config(TIMER_FREQ / OS_TIMER_TICKS_PER_SEC)) {
        while (1) _NOP();
    }
 }

 void SysTick_update(void)
 {
    *(uint64_t*)(TIMER_CTRL_ADDR + TIMER_MTIME) = 0;
 }

 void LED_init(void)
 {
    rcu_periph_clock_enable(RCU_GPIOA);
    gpio_init(LED_PORT, GPIO_MODE_OUT_PP, GPIO_OSPEED_50MHZ, LED_PIN);
    gpio_bit_reset(LED_PORT, LED_PIN);
 }

 void LED_toggle(void)
 {
    gpio_bit_write(LED_PORT, LED_PIN, (bit_status)(1-gpio_input_bit_get(LED_PORT, LED_PIN)));
 }
#endif

/*************************************************************/

#ifdef _OS_SAML10_ 
 #define PORTA         0                                    // user-LED definition
 #define LED_GROUP     PORTA                                // -- TAU board used for SAML10
 #define LED_PORT      PORT_PA27
 #define LED_PIN       PIN_PA27

 void SysTick_init(void)
 {
    PM->INTFLAG.reg = PM_INTFLAG_PLRDY;
    PM->PLCFG.reg = PM_PLCFG_PLSEL_PL2;
    while (0 == PM->INTFLAG.bit.PLRDY);
    OSCCTRL->OSC16MCTRL.reg = OSCCTRL_OSC16MCTRL_ENABLE | OSCCTRL_OSC16MCTRL_FSEL_16;
    // ---------------------
    SystemCoreClockUpdate();
    if (SysTick_Config(SystemCoreClock / OS_TIMER_TICKS_PER_SEC)) {
        while (1) __NOP();
    }
 }

 void LED_init(void)
 {
    PORT->Group[LED_GROUP].DIRSET.reg = LED_PORT;
    PORT->Group[LED_GROUP].OUTSET.reg = LED_PORT;
 }

 void LED_toggle(void)
 {
    PORT->Group[LED_GROUP].OUTTGL.reg = LED_PORT;
 }
#endif

/*************************************************************/

#ifdef _OS_SAML11_ 
 #define PORTA         0                                    // user-LED definition
 #define LED_GROUP     PORTA                                // -- TAU board used for SAML11
 #define LED_PORT      PORT_PA27
 #define LED_PIN       PIN_PA27

 void SysTick_init(void)
 {
    // switch to 16MHz is done is secure "main_s.c"

    // ---------------------
    SystemCoreClockUpdate();
    if (SysTick_Config(SystemCoreClock / OS_TIMER_TICKS_PER_SEC)) {
        while (1) __NOP();
    }
 }

 void LED_init(void)
 {
    PORT->Group[LED_GROUP].DIRSET.reg = LED_PORT;
    PORT->Group[LED_GROUP].OUTSET.reg = LED_PORT;
 }

 void LED_toggle(void)
 {
    PORT->Group[LED_GROUP].OUTTGL.reg = LED_PORT;
 }
#endif

/*************************************************************/

#ifdef _OS_SAMD21_

 #define SAMD21_TAU        0
 #define SAMD21_SPARKFUN   1
 #define SAMD21_15x15      2

 #define SAMD21_TYPE       SAMD21_TAU

 #define PORTA         0                       // user-LED definition
 #define LED_GROUP     PORTA
 #if (SAMD21_TYPE == SAMD21_TAU)               // -- TAU board
  #define LED_PORT     PORT_PA27
  #define LED_PIN      PIN_PA27
 #endif
 #if (SAMD21_TYPE == SAMD21_SPARKFUN)          // -- Sparkfun
  #define LED_PORT     PORT_PA17
  #define LED_PIN      PIN_PA17
 #endif
 #if (SAMD21_TYPE == SAMD21_15x15)             // -- SAMD15x15
  #define LED_PORT     PORT_PA09
  #define LED_PIN      PIN_PA09
 #endif

 void SysTick_init(void)
 {
    SYSCTRL->XOSC32K.reg  = SYSCTRL_XOSC32K_ENABLE;
    SYSCTRL->XOSC32K.reg |= SYSCTRL_XOSC32K_STARTUP(5) | SYSCTRL_XOSC32K_AAMPEN | SYSCTRL_XOSC32K_EN32K | SYSCTRL_XOSC32K_XTALEN | SYSCTRL_XOSC32K_ENABLE;
    while((SYSCTRL->PCLKSR.reg & SYSCTRL_PCLKSR_XOSC32KRDY) == 0) __NOP();
    GCLK->GENDIV.reg  = GCLK_GENDIV_DIV(1) | GCLK_GENDIV_ID(1);
    GCLK->GENCTRL.reg = GCLK_GENCTRL_ID(1) | GCLK_GENCTRL_SRC_XOSC32K | GCLK_GENCTRL_GENEN;
    GCLK->CLKCTRL.reg = GCLK_CLKCTRL_GEN_GCLK1 | GCLK_CLKCTRL_CLKEN | GCLK_CLKCTRL_ID(0) ;
    SYSCTRL->DFLLCTRL.reg  = SYSCTRL_DFLLCTRL_ENABLE;
    SYSCTRL->DFLLMUL.reg   = SYSCTRL_DFLLMUL_CSTEP(5) | SYSCTRL_DFLLMUL_FSTEP(10); 	
    SYSCTRL->DFLLMUL.reg  |= SYSCTRL_DFLLMUL_MUL(1465); // 1465 * 32768 is approx 48MHZ
    SYSCTRL->DFLLCTRL.reg |= SYSCTRL_DFLLCTRL_MODE;
    while((SYSCTRL->PCLKSR.reg & SYSCTRL_PCLKSR_DFLLLCKC) == 0) __NOP();
    NVMCTRL->CTRLB.reg |= NVMCTRL_CTRLB_RWS(1);
    GCLK->GENDIV.reg  = GCLK_GENDIV_DIV(1) | GCLK_GENDIV_ID(0);
    GCLK->GENCTRL.reg = GCLK_GENCTRL_GENEN | GCLK_GENCTRL_SRC_DFLL48M | GCLK_GENCTRL_ID(0);
    GCLK->CLKCTRL.reg = GCLK_CLKCTRL_CLKEN | GCLK_CLKCTRL_GEN_GCLK0 | GCLK_CLKCTRL_ID_DFLL48;
    if (SysTick_Config(SystemCoreClock / OS_TIMER_TICKS_PER_SEC)) {
        while (1) __NOP();
    }
 }

 void LED_init(void)
 {
    PORT->Group[LED_GROUP].DIRSET.reg = LED_PORT;
    PORT->Group[LED_GROUP].OUTSET.reg = LED_PORT;
 }

 void LED_toggle(void)
 {
    PORT->Group[LED_GROUP].OUTTGL.reg = LED_PORT;
 }
#endif

/*************************************************************/

#ifdef _OS_SAMD51_

 #define SAMD51_TACHYON        0
 #define SAMD51_ITSYBITSY_M4   1

 #define SAMD51_TYPE           SAMD51_ITSYBITSY_M4

 #define PORTA         0                       // user-LED definition
 #define LED_GROUP     PORTA
 #if (SAMD51_TYPE == SAMD51_TACHYON)           // -- TACHYON board
  #define LED_PORT     PORT_PA27
  #define LED_PIN      PIN_PA27
 #endif
 #if (SAMD51_TYPE == SAMD51_ITSYBITSY_M4)      // -- Adafruit ItsyBitsy M4 Express
  #define LED_PORT     PORT_PA22
  #define LED_PIN      PIN_PA22
 #endif

 void SysTick_init(void)
 {
    NVMCTRL->CTRLA.bit.RWS = 5;
    GCLK->CTRLA.bit.SWRST = 1;
    while (GCLK->SYNCBUSY.reg & GCLK_SYNCBUSY_SWRST) _NOP();
    GCLK->GENCTRL[3].reg = GCLK_GENCTRL_SRC(GCLK_GENCTRL_SRC_OSCULP32K) | GCLK_GENCTRL_GENEN;
    while (GCLK->SYNCBUSY.reg & GCLK_SYNCBUSY_GENCTRL3) _NOP();
    GCLK->GENCTRL[0].reg = GCLK_GENCTRL_SRC(GCLK_GENCTRL_SRC_OSCULP32K) | GCLK_GENCTRL_GENEN;
    while (GCLK->SYNCBUSY.reg & GCLK_SYNCBUSY_GENCTRL0) _NOP();
    OSCCTRL->DFLLCTRLA.reg = 0;
    //GCLK->PCHCTRL[OSCCTRL_GCLK_ID_DFLL48].reg = (1 << GCLK_PCHCTRL_CHEN_Pos) | GCLK_PCHCTRL_GEN(GCLK_PCHCTRL_GEN_GCLK3_Val);
    OSCCTRL->DFLLMUL.reg = OSCCTRL_DFLLMUL_CSTEP(1) | OSCCTRL_DFLLMUL_FSTEP(1) | OSCCTRL_DFLLMUL_MUL(0);
    while (OSCCTRL->DFLLSYNC.reg & OSCCTRL_DFLLSYNC_DFLLMUL) _NOP();
    OSCCTRL->DFLLCTRLB.reg = 0;
    while (OSCCTRL->DFLLSYNC.reg & OSCCTRL_DFLLSYNC_DFLLCTRLB) _NOP();
    OSCCTRL->DFLLCTRLA.reg |= OSCCTRL_DFLLCTRLA_ENABLE;
    while (OSCCTRL->DFLLSYNC.reg & OSCCTRL_DFLLSYNC_ENABLE) _NOP();
    OSCCTRL->DFLLVAL.reg = OSCCTRL->DFLLVAL.reg;
    while (OSCCTRL->DFLLSYNC.bit.DFLLVAL) _NOP();
    OSCCTRL->DFLLCTRLB.reg = OSCCTRL_DFLLCTRLB_WAITLOCK | OSCCTRL_DFLLCTRLB_CCDIS | OSCCTRL_DFLLCTRLB_USBCRM ;
    while (!OSCCTRL->STATUS.bit.DFLLRDY) _NOP();
    GCLK->GENCTRL[5].reg = GCLK_GENCTRL_SRC(GCLK_GENCTRL_SRC_DFLL_Val) | GCLK_GENCTRL_GENEN | GCLK_GENCTRL_DIV(24u);
    while (GCLK->SYNCBUSY.bit.GENCTRL5) _NOP();
    GCLK->PCHCTRL[OSCCTRL_GCLK_ID_FDPLL0].reg = (1 << GCLK_PCHCTRL_CHEN_Pos) | GCLK_PCHCTRL_GEN(GCLK_PCHCTRL_GEN_GCLK5_Val);
    OSCCTRL->Dpll[0].DPLLRATIO.reg = OSCCTRL_DPLLRATIO_LDRFRAC(0x00) | OSCCTRL_DPLLRATIO_LDR(59);
    while (OSCCTRL->Dpll[0].DPLLSYNCBUSY.bit.DPLLRATIO) _NOP();
    OSCCTRL->Dpll[0].DPLLCTRLB.reg = OSCCTRL_DPLLCTRLB_REFCLK_GCLK | OSCCTRL_DPLLCTRLB_LBYPASS;
    OSCCTRL->Dpll[0].DPLLCTRLA.reg = OSCCTRL_DPLLCTRLA_ENABLE;
    while ((OSCCTRL->Dpll[0].DPLLSTATUS.bit.CLKRDY == 0) || (OSCCTRL->Dpll[0].DPLLSTATUS.bit.LOCK == 0)) _NOP();
    GCLK->PCHCTRL[OSCCTRL_GCLK_ID_FDPLL1].reg = (1 << GCLK_PCHCTRL_CHEN_Pos) | GCLK_PCHCTRL_GEN(GCLK_PCHCTRL_GEN_GCLK5_Val);
    OSCCTRL->Dpll[1].DPLLRATIO.reg = OSCCTRL_DPLLRATIO_LDRFRAC(0x00) | OSCCTRL_DPLLRATIO_LDR(49);
    while (OSCCTRL->Dpll[1].DPLLSYNCBUSY.bit.DPLLRATIO) _NOP();
    OSCCTRL->Dpll[1].DPLLCTRLB.reg = OSCCTRL_DPLLCTRLB_REFCLK_GCLK | OSCCTRL_DPLLCTRLB_LBYPASS;
    OSCCTRL->Dpll[1].DPLLCTRLA.reg = OSCCTRL_DPLLCTRLA_ENABLE;
    while ((OSCCTRL->Dpll[1].DPLLSTATUS.bit.CLKRDY == 0) || (OSCCTRL->Dpll[1].DPLLSTATUS.bit.LOCK == 0)) _NOP();
    GCLK->GENCTRL[1].reg = GCLK_GENCTRL_SRC(GCLK_GENCTRL_SRC_DFLL_Val) | GCLK_GENCTRL_IDC | GCLK_GENCTRL_GENEN;
    while (GCLK->SYNCBUSY.reg & GCLK_SYNCBUSY_GENCTRL1) _NOP();
    GCLK->GENCTRL[2].reg = GCLK_GENCTRL_SRC(GCLK_GENCTRL_SRC_DPLL1_Val) | GCLK_GENCTRL_IDC | GCLK_GENCTRL_GENEN;
    while (GCLK->SYNCBUSY.reg & GCLK_SYNCBUSY_GENCTRL2) _NOP();
    GCLK->GENCTRL[0].reg = GCLK_GENCTRL_SRC(GCLK_GENCTRL_SRC_DPLL0) | GCLK_GENCTRL_IDC | GCLK_GENCTRL_GENEN;
    while (GCLK->SYNCBUSY.reg & GCLK_SYNCBUSY_GENCTRL0) _NOP();
    MCLK->CPUDIV.reg = MCLK_CPUDIV_DIV_DIV1;
    SUPC->VREG.bit.SEL = 0; 
    // ---------------------
    SystemCoreClockUpdate();
    if (SysTick_Config(120000000 / OS_TIMER_TICKS_PER_SEC)) {
        while (1) __NOP();
    }
 }

 void SysTick_update(void)
 {
 }

 void LED_init(void)
 {
    PORT->Group[LED_GROUP].DIRSET.reg = LED_PORT;
    PORT->Group[LED_GROUP].OUTSET.reg = LED_PORT;
 }

 void LED_toggle(void)
 {
    PORT->Group[LED_GROUP].OUTTGL.reg = LED_PORT;
 }
#endif
 
/*************************************************************/

#ifdef _OS_STM32F1xx_
 #include "stm32f10x_rcc.h"
 #include "stm32f10x_gpio.h"

 #define   GCLK_CLKCTRL_ID_DFLL48_Val   0x0ul
 #define   GCLK_CLKCTRL_ID_DFLL48      (GCLK_CLKCTRL_ID_DFLL48_Val    << GCLK_CLKCTRL_ID_Pos)

 void SysTick_init(void)
 {
    SystemCoreClockUpdate();
    if (SysTick_Config(SystemCoreClock / OS_TIMER_TICKS_PER_SEC)) {
        while (1) __NOP();
    }
 }

 void LED_init(void)
 {
    GPIO_InitTypeDef GPIO_InitStructure;                   // user-LED PA1 (HAOYU HY-TinySTM103T)

    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_Init(GPIOA, &GPIO_InitStructure);
    GPIO_Write(GPIOA, GPIO_Pin_1);
    GPIO_Write(GPIOA, (U16)~GPIO_ReadOutputData(GPIOA));   // switch ON (control)
 }

 void LED_toggle(void)
 {
    GPIO_Write(GPIOA, (U16)~GPIO_ReadOutputData(GPIOA));
 }
#endif
 
/*************************************************************/

#ifdef _OS_STM32F4xx_
 #include "stm32f4xx_rcc.h"
 #include "stm32f4xx_gpio.h"

 void SysTick_init(void)
 {
    SystemCoreClockUpdate();
    if (SysTick_Config(SystemCoreClock / OS_TIMER_TICKS_PER_SEC)) {
        while (1) __NOP();
    }
 }

 void LED_init(void)
 {
    GPIO_InitTypeDef GPIO_InitStructure;

    RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOF, ENABLE);  // state LED PF6 (HAOYU HY-STM32F4xxCore144)
    GPIO_InitStructure.GPIO_Pin   = GPIO_Pin_6;
    GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_NOPULL;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_OUT;
    GPIO_Init(GPIOF, &GPIO_InitStructure);
    GPIO_ToggleBits(GPIOF, GPIO_Pin_6);                    // switch ON (control)
 }

 void LED_toggle(void)
 {
    GPIO_ToggleBits(GPIOF, GPIO_Pin_6);
 }
#endif

/*************************************************************/

#ifdef _OS_FE310_
 #define RTC_FREQ 32768
 void SysTick_init(void)
 {
    clear_csr(mie, MIP_MEIP);           // extern PLIC
    clear_csr(mie, MIP_MTIP);           // Timer
    clear_csr(mie, MIP_MSIP);           // Software
    CLINT->mtimecmp = CLINT->mtime + (RTC_FREQ / OS_TIMER_TICKS_PER_SEC);
    set_csr(mie, MIP_MTIP);             // Timer
    set_csr(mstatus, MSTATUS_MIE);      // General
 }

 void SysTick_update(void)
 {
    clear_csr(mie, MIP_MTIP);           // Timer
    CLINT->mtimecmp = CLINT->mtime + (RTC_FREQ / OS_TIMER_TICKS_PER_SEC);
    set_csr(mie, MIP_MTIP);             // Timer
 }

 void LED_init(void)
 {
    GPIO0->iof_en    &= ~GPIO_pin(19);
    GPIO0->out_xor   &= ~GPIO_pin(19);
    GPIO0->output_en |= GPIO_pin(19);
 }

 void LED_toggle(void)
 {
    if(GPIO0->port & GPIO_pin(19))
        GPIO0->port &= ~GPIO_pin(19);
    else
        GPIO0->port |= GPIO_pin(19);
 }

 #define F_CPU 320000000                 // mpu PLL-clock to use (320MHz)
 extern void trap_entry(void);
 void SystemInit(void)
 {
    clear_csr(mie, MIP_MEIP);
    write_csr(mtvec, &trap_entry);

 #ifdef F_CPU	// PLL
    RTC->lfrosccfg  &= ~RTC_lfrosccfg_en;
    PRCI->hfrosccfg  = PRCI_hfrosccfg_hfroscdiv_s(4) | PRCI_hfrosccfg_hfrosctrim_s(16) | PRCI_hfrosccfg_hfroscen;
    while((PRCI->hfrosccfg & PRCI_hfrosccfg_hfroscrdy) == 0);
    PRCI->pllcfg    &= ~PRCI_pllcfg_pllsel;
    SPI0->sckdiv     = 8;               // Slow down the QSPI
    PRCI->plloutdiv  = PRCI_plloutdiv_plloutdivby1 | PRCI_plloutdiv_plloutdiv_s(0);
    PRCI->pllcfg     = PRCI_pllcfg_pllrefsel | PRCI_pllcfg_pllbypass | PRCI_pllcfg_pllr_s(1) | PRCI_pllcfg_pllf_s(PLL_F_VAL) | PRCI_pllcfg_pllq_s(1);
    PRCI->pllcfg    &= ~PRCI_pllcfg_pllbypass;
    uint64_t now = CLINT->mtime;
    while (CLINT->mtime - now < 4);     // wait 100us (4 ticks of RTC)
    while((PRCI->pllcfg & PRCI_pllcfg_plllock) == 0);
    PRCI->pllcfg    |= PRCI_pllcfg_pllsel;
 #endif
 }
#endif

/*************************************************************/

#ifdef _OS_ARC_EM_
 #define INT_CTRL_IE         (1 << 4)
 #define TIMER_CTRL_IE       (1 << 0)
 #define TIMER_CTRL_NH       (1 << 1)
 #define TIMER_CTRL_IP       (1 << 3)
 #define EXC_NO_TIMER0        16
 #define TIMER0_FREQ          25000000
 void SysTick_init(void)
 {
    _sr(0, ARC_REG_TIMER0_CTRL);
    _sr(TIMER0_FREQ / OS_TIMER_TICKS_PER_SEC, ARC_REG_TIMER0_LIMIT);
    _sr(0, ARC_REG_TIMER0_CNT);
    _sr(TIMER_CTRL_IE | TIMER_CTRL_NH, ARC_REG_TIMER0_CTRL);
 }

 void SysTick_update(void)
 {
    _sr(_lr(ARC_REG_TIMER0_CTRL) & ~TIMER_CTRL_IP, ARC_REG_TIMER0_CTRL);
 }

 typedef struct port_ctrl {
    U32 DR;
    U32 DDR;
    U32 CTRL;
 } PORT_CTRL;
 typedef volatile struct dw_gpio_reg {
    PORT_CTRL SWPORTS[4];
 } *LED_REG_PTR;
 #define REGBASE_GPIO0           (0x00002000U)   /*!< GPIO 0 Onboard */
 LED_REG_PTR led_reg_ptr = (LED_REG_PTR)(0xF0000000U + REGBASE_GPIO0);

 void LED_init(void)
 {
    led_reg_ptr->SWPORTS[1].DR  = 0xFF;
    led_reg_ptr->SWPORTS[1].DDR = 0xFF;
 }

 void LED_toggle(void)
 {
    if(led_reg_ptr->SWPORTS[1].DR & 1) led_reg_ptr->SWPORTS[1].DR &= ~1;
    else                               led_reg_ptr->SWPORTS[1].DR |=  1;
 }

 void SystemInit(void)
 {
 #ifndef _ARC_SECURESHIELD
    U32 irq_ctrl = (1 << 13) |         // LP        auto-save/restore code-density registers
                   (1 << 11) |         // U         user-context to user-stack
                   (1 << 10) |         // L         auto-save/restore loop registers
                   (1 <<  9) |         // B         auto-save/restore BLINK
  #ifndef _ARC_RF16
                   (16);               // NR[4:0]   16 pairs -> all of the 32 registers
  #else
                   (8);                // NR[4:0]    8 pairs -> all of the 16 registers
  #endif
    _seti(INT_CTRL_IE | 2);
    _sr(irq_ctrl, ARC_REG_IRQ_CTRL);
 #else
                                       // on SecureShield configurations, the AUX_IRQ_CTRL have to be written
                                       // on secure-startup and the used interrupts (16 & 18) have to be
                                       // configured to be usable here - take care timer-0 interrupt is off !
    _sr(0, ARC_REG_TIMER0_CTRL);
 #endif
 }
#endif

/*************************************************************/

#ifdef _OS_M2351_
 void SysTick_init(void)
 {
    // switch to 64MHz is done is secure "main_s.c"
    // ---------------------
    SystemCoreClock = 64000000;
    if (SysTick_Config(SystemCoreClock / OS_TIMER_TICKS_PER_SEC)) {
        while (1) __NOP();
    }
 }

 void SysTick_update(void)
 {
 }

 void LED_init(void)
 {
    PA_NS->MODE = (PA_NS->MODE & (~(3ul << 11*2))) | GPIO_MODE_OUTPUT << 11*2;
 }

 void LED_toggle(void)
 {
    PA11_NS ^= 1;
 }
#endif

/*************************************************************/

#ifdef _OS_AT91SAM3xxx_
 extern uint32_t SystemFrequency;

 void SysTick_init(void)
 {
    SysTick_Config(SystemFrequency/OS_TIMER_TICKS_PER_SEC);// Generate interrupt each ~10 ms
 }

 void LED_init(void)
 {
 }

 void LED_toggle(void)
 {
 }
#endif

/*************************************************************/

U32 OS_SystemTicksCnt;

OS_STK_TYPE Stk1[TASK_STK_SIZE];
OS_STK_TYPE Stk2[TASK_STK_SIZE];

SFS_USER SFS_T1, SFS_T2;

void  Task1(void *Tdata) KC51_REENTRANT;
void  Task2(void *Tdata) KC51_REENTRANT;

int main(void)
{
    // ---- HW specific ----
    SystemInit();
    // ---------------------

    OS_Init();
#if OS_STK_CHECK_EN
    OS_TaskCreate(Task1, NULL, (void *)&Stk1[TASK_STK_SIZE], (void *)&Stk1, 2);
    OS_TaskCreate(Task2, NULL, (void *)&Stk2[TASK_STK_SIZE], (void *)&Stk2, 3);
#else
    OS_TaskCreate(Task1, NULL, (void *)&Stk1[TASK_STK_SIZE], 2);
    OS_TaskCreate(Task2, NULL, (void *)&Stk2[TASK_STK_SIZE], 3);
#endif

    OS_Start();
    while(1);
}

void OSTickISR(void)                                       // typical pC/OS ISR
{
#if !defined(_OS_FE310_) && !defined(_OS_ARC_EM_) && !defined(_OS_GD32VF103C_)
    OS_IntEnter();
#else
    // ---- HW specific ----
    SysTick_update();
    // ---------------------
#endif

#if OS_TMR_EN
    OSTimerTick();                                         // call pC/OS-function (timers)
#endif
    if((++OS_SystemTicksCnt) == OS_SYSTEM_TICKS_DIV) {
        OS_SystemTicksCnt = 0;
        OSSystemTick();                                    // call pC/OS-function (delays / wait)
    }

#if !defined(_OS_FE310_) && !defined(_OS_ARC_EM_) && !defined(_OS_GD32VF103C_)
    OS_IntExit();
#endif
}

void AppTickInit(void)
{                                                          // Configure the system tick
    OS_SystemTicksCnt = 0;
    
    // ---- HW specific ----
    SysTick_init();
    // ---------------------
}

U08 Fill[]="______________________________________________";
U08 Text[]="SFS_Test_File R/W";

void Task1(void *Tdata) KC51_REENTRANT
{
    U08            returnOk, err;
    U08            Readed[20];
    U08 OS_HUGE   *OS_Rev;
    SFS_HANDLE     fhandle = 0;
    SFS_LONG       written;
    SFS_ATTR       a;
    SFS_GET        fentry;

    Tdata = Tdata;

    AppTickInit();

    // ---- HW specific ----
    LED_init();
    // ---------------------

    OS_Rev = OS_GetRev();
    OS_Rev = OS_Rev;
    
    SFS_Init();                        // init SerialFileSystem

    returnOk = SFS_Format();
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_BecomeUser(&SFS_T1);
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_ChangeDir("/");
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_CreateDir("Dir1");
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_ChangeDir("Dir1");
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_ChangeDir("./");
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_ChangeDir("/");
    if(returnOk==SFS_NO_ERR) {
        returnOk=SFS_CreateFile("Dir1/File11", SFS_ATTR_RW, 100);
        returnOk=SFS_CreateFile("Dir1/File12", SFS_ATTR_RW, 200);
        returnOk=SFS_CreateFile("Dir1/File13", SFS_ATTR_RW, 300);
        returnOk=SFS_CreateFile("Dir1/File14", SFS_ATTR_RW, 400);
        returnOk=SFS_CreateFile("Dir1/File15", SFS_ATTR_RW, 500);
        returnOk=SFS_CreateFile("Dir1/File16", SFS_ATTR_RW, 600);
        returnOk=SFS_CreateFile("Dir1/File17", SFS_ATTR_RW, 700);
        returnOk=SFS_CreateFile("Dir1/File18", SFS_ATTR_RW, 800);
        returnOk=SFS_CreateFile("Dir1/File19", SFS_ATTR_RW, 900);
        returnOk=SFS_CreateFile("Dir1/File20", SFS_ATTR_RW, 1000);
        returnOk=SFS_CreateFile("Dir1/File21", SFS_ATTR_RW, 1100);
        err     =SFS_CreateFile("Dir1/File22", SFS_ATTR_RW, 1200);
        if(err!=SFS_NO_ERR)
            err =SFS_CreateFile("Dir1/File23", SFS_ATTR_RW, 1300);
    }
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_CreateFile("File1", SFS_ATTR_RO | SFS_ATTR_RW, 100);
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_CreateDir("/Dir2");
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_CreateFile("Dir2/File21", SFS_ATTR_RO, 10);
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_CreateFile("/Dir2/File22", SFS_ATTR_RW, 300);

    if(returnOk==SFS_NO_ERR) {
        returnOk=SFS_CreateDir("DirT");
        if(returnOk==SFS_NO_ERR) {
            returnOk=SFS_CreateFile("DirT/FileT1", SFS_ATTR_RW, 100);
            returnOk=SFS_CreateDir("DirT/DirT1");
            returnOk=SFS_CreateFile("DirT/FileT2", SFS_ATTR_RW, 100);
            returnOk=SFS_CreateFile("DirT/DirT1/FileT11", SFS_ATTR_RW, 100);
            returnOk=SFS_CreateDir("DirT/DirT1/DirT11");
            returnOk=SFS_CreateDir("DirT/DirT1/DirT12");
            returnOk=SFS_CreateFile("DirT/DirT1/DirT12/FileT121", SFS_ATTR_RW, 100);
            returnOk=SFS_CreateFile("DirT/DirT1/FileT12", SFS_ATTR_RW, 100);
            returnOk=SFS_CreateLink("Dir2/File21", "DirT/DirT1/LinkT11");
            returnOk=SFS_CreateFile("DirT/DirT1/DirT11/FileT111", SFS_ATTR_RW, 100);
            returnOk=SFS_CreateDir("DirT/DirT1/DirT11/DirT111");
            returnOk=SFS_CreateFile("DirT/DirT1/DirT11/FileT112", SFS_ATTR_RW, 100);
            returnOk=SFS_CreateFile("DirT/DirT1/DirT11/DirT111/FileT1111", SFS_ATTR_RW, 100);
            returnOk=SFS_CreateLink("DirT/DirT1/DirT11/DirT111", "DirT/DirT1/DirT12/LinkT121");
            returnOk=SFS_CreateLink("DirT/DirT1/DirT12/LinkT121", "DirT/DirT1/DirT12/LinkT122");
            returnOk=SFS_CreateLink("DirT/DirT1/DirT12/LinkT122", "DirT/DirT1/DirT12/LinkT123");
            returnOk=SFS_CreateLink("DirT/DirT1/DirT12/LinkT123", "DirT/DirT1/DirT12/LinkT124");
            returnOk=SFS_CreateLink("DirT/DirT1/DirT12/LinkT124", "DirT/DirT1/DirT12/LinkT125");
            returnOk=SFS_CreateLink("DirT/DirT1/DirT11/DirT111", "DirT/DirT1/DirT11/DirT111/LinkT1111");
            returnOk=SFS_CreateLink("DirT/DirT1/FileT12", "DirT/DirT1/DirT11/LinkT111");
            returnOk=SFS_CreateLink("DirT/DirT1/FileT12", "DirT/DirT1/DirT11/LinkT112");
#if 1
            returnOk=SFS_CreateLink("DirT/DirT1/FileT12", "Dir2/LinkT21");
            err     =SFS_RemoveDirTree("DirT/DirT1");
            returnOk=SFS_RemoveLink("Dir2/LinkT21");
            returnOk=SFS_CreateDir("DirT/DirT1/DirT12");
            returnOk=SFS_CreateFile("DirT/DirT1/DirT12/FileT121", SFS_ATTR_RW, 100);
            returnOk=SFS_CreateLink("DirT/DirT1/DirT12/FileT121", "Dir2/LinkT22");
            err     =SFS_RemoveDirTree("DirT/DirT1");
            returnOk=SFS_RemoveLink("Dir2/LinkT22");
#endif
            if(returnOk==SFS_NO_ERR) {
                returnOk=SFS_RemoveDirTree("DirT/DirT1");
                if(returnOk==SFS_NO_ERR) {
                    err=SFS_ChangeDir("DirT/DirT1");
                }
            }
        }
    }

    if(returnOk==SFS_NO_ERR) {
        returnOk=SFS_CreateDir("/Dir3");
        returnOk=SFS_CreateDir("/Dir3/Dir31");
        returnOk=SFS_CreateDir("/Dir3/Dir32");
        returnOk=SFS_CreateFile("/Dir3/Dir31/File311", SFS_ATTR_RW, 200);
        returnOk=SFS_CreateLink("/Dir3", "/DirL1");
        returnOk=SFS_CreateLink("/DirL1", "/DirL2");
        returnOk=SFS_CreateLink("/DirL2", "/DirL3");
        returnOk=SFS_CreateLink("/Dir3/Dir31/File311", "/Dir3/Dir32/FileL1");
        returnOk=SFS_CreateLink("/Dir3/Dir32/FileL1", "/Dir3/Dir32/FileL2");
        returnOk=SFS_CreateLink("/Dir3/Dir32/FileL2", "/Dir3/Dir32/FileL3");
        if(returnOk==SFS_NO_ERR) {
            fhandle=SFS_OpenFile("/Dir3/Dir32/FileL3", SFS_ACC_RW);
            if(!(fhandle))
                returnOk=SFS_GetErrNo();
            else
                returnOk=SFS_CloseFile(fhandle);
        }
        if(returnOk==SFS_NO_ERR) {
            fhandle=SFS_OpenFile("/Dir3/Dir32/FileL3", SFS_ACC_RW);
            if(!(fhandle))
                returnOk=SFS_GetErrNo();
            else
                returnOk=SFS_CloseFile(fhandle);
        }
        if(returnOk==SFS_NO_ERR) {
            fhandle=SFS_OpenFile("/DirL3/Dir32/FileL3", SFS_ACC_RW);
            if(!(fhandle))
                returnOk=SFS_GetErrNo();
            else
                returnOk=SFS_CloseFile(fhandle);
        }
        if(returnOk==SFS_NO_ERR) {
            returnOk=SFS_ChangeDir("/DirL3");
            if(returnOk==SFS_NO_ERR) {
                fhandle=SFS_OpenFile("Dir32/FileL3", SFS_ACC_RW);
                if(!(fhandle))
                    returnOk=SFS_GetErrNo();
                else
                    returnOk=SFS_CloseFile(fhandle);
                returnOk=SFS_ChangeDir("..");
            }
        }
    }

    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_ChangeDir("/Dir2");
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_CreateDir("Dir22");
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_CreateDir("../Dir2/Dir21");
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_ChangeDir("Dir21");
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_CreateFile("/Dir2/Dir21/File211", SFS_ATTR_RO, 200);
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_CreateFile("/Dir2/Dir21/RW_test", SFS_ATTR_RO, 50);
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_CreateFile("/Dir2/Dir21/RW_test2", SFS_ATTR_RW, 400);
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_CreateFile("/Dir2/Dir21/File212", SFS_ATTR_RW, 150);
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_ChangeDir("..");
    if(returnOk==SFS_NO_ERR) {
        returnOk=SFS_RemoveFile("/Dir1/File11");
        returnOk=SFS_RemoveFile("/Dir1/File13");
        returnOk=SFS_RemoveFile("/Dir1/File15");
        returnOk=SFS_RemoveFile("/Dir1/File17");
        returnOk=SFS_RemoveFile("/Dir1/File19");
        returnOk=SFS_RemoveFile("/Dir1/File21");
        err     =SFS_RemoveFile("/Dir1/File23");
    }
    
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_CreateDir("Dir21/Dir211");
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_CreateDir("Dir21/Dir211/Dir2111");
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_CreateDir("Dir21/Dir211/Dir2111/Dir21111");
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_CreateFile("Dir21/Dir211/Dir2111/File21111", SFS_ATTR_RW, 100);
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_CreateFile("Dir21/Dir211/Dir2111/Dir21111/File21111", SFS_ATTR_RW, 200);
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_CreateLink("Dir21/Dir211/Dir2111", "/Dir1/DirL11");
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_CreateLink("Dir21/Dir211/Dir2111/File21111", "/Dir1/FileL21111");
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_CreateLink("Dir21/Dir211/Dir2111/Dir21111/File21111", "/FileL21111");

    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_GetEntry("Dir21/Dir211/Dir2111", &fentry);
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_GetEntry("Dir21/Dir211/Dir2111/File21111", &fentry);
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_GetEntry("/Dir1/FileL21111", &fentry);
    
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_ChangeDirTemp("/Dir1/DirL11/Dir21111");
    if(returnOk==SFS_NO_ERR) {
        fhandle=SFS_OpenFile("/Dir1/FileL21111", SFS_ACC_RW);
        if(!(fhandle))
            returnOk=SFS_GetErrNo();
        else
            returnOk=SFS_CloseFile(fhandle);
    }
    if(returnOk==SFS_NO_ERR) {
        fhandle=SFS_OpenFile("File21111", SFS_ACC_RW);
        if(!(fhandle))
            returnOk=SFS_GetErrNo();
        else
            returnOk=SFS_CloseFile(fhandle);
    }
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_BackDirTemp();
    if(returnOk==SFS_NO_ERR)
        err=SFS_RemoveFile("Dir21/Dir211/Dir2111/File21111");
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_RemoveLink("/Dir1/FileL21111");
    if(returnOk==SFS_NO_ERR)
        err=SFS_RemoveFile("/FileL21111");
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_RemoveFile("Dir21/Dir211/Dir2111/File21111");
    if(returnOk==SFS_NO_ERR)
        err=SFS_RemoveDir("/Dir1/DirL11");
    
    OS_TimeDly(5);
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_CreateFile("../File2", SFS_ATTR_RW, 500);
    if(returnOk==SFS_NO_ERR) {
        returnOk=SFS_AttribFile("Dir21/RW_test", SFS_ATTR_RW);
        if (returnOk==SFS_NO_ERR) {
            a = SFS_GetFileAttrib("/Dir2/Dir21/RW_test");
            if ((a & SFS_ATTR_RW) != SFS_ATTR_RW)
                returnOk=SFS_GetErrNo();
            fhandle=SFS_OpenFile("/Dir2/Dir21/RW_test", SFS_ACC_RW);
            if(!(fhandle))
                returnOk=SFS_GetErrNo();
        }
        if(returnOk==SFS_NO_ERR) {
            written=SFS_WriteFile(Fill , sizeof(Fill), fhandle);
            if(written != sizeof(Fill))
                returnOk=SFS_GetErrNo();
            if(returnOk==SFS_NO_ERR)
                returnOk=SFS_SeekFile(45, fhandle);
            if(returnOk==SFS_NO_ERR) {
                written=SFS_WriteFile(Text , sizeof(Text), fhandle);
                if(written != sizeof(Text))
                    returnOk=SFS_GetErrNo();
                if(returnOk==SFS_NO_ERR) {
                    returnOk=SFS_SeekFile(45, fhandle);
                    if(returnOk==SFS_NO_ERR) {
                        written=SFS_ReadFile((U08 OS_HUGE *)Readed, sizeof(Text), fhandle);
                        if(written != sizeof(Text))
                            returnOk=SFS_GetErrNo();
                    }
                }
            }
            returnOk=SFS_CloseFile(fhandle);
            if(returnOk==SFS_NO_ERR)
                returnOk=SFS_RenameFile("/Dir2/Dir21/RW_test","config");
        }
    }

    while(1) {
        fhandle = SFS_OpenFile("/Dir2/Dir21/config", SFS_ACC_RW);
        if(fhandle) {
            returnOk = SFS_SeekFile(45, fhandle);
            if(returnOk == SFS_NO_ERR) {
                written = SFS_WriteFile(Text , sizeof(Text), fhandle);
                if(written != sizeof(Text))
                    returnOk = SFS_GetErrNo();
                if(returnOk == SFS_NO_ERR) {
                    returnOk = SFS_SeekFile(45, fhandle);
                    if(returnOk == SFS_NO_ERR) {
                        written = SFS_ReadFile((U08 OS_HUGE *)Readed, sizeof(Text), fhandle);
                        if(written != sizeof(Text))
                            returnOk=SFS_GetErrNo();
                        else if(0 != SFSstrcmp((SFS_PATHNAME OS_HUGE *)Text, (SFS_PATHNAME OS_HUGE *)Readed))
                            while(1);
                    }
                }
            }
            returnOk = SFS_CloseFile(fhandle);
            // ---- HW specific ----
            LED_toggle();
            // ---------------------
        } else {
            returnOk = SFS_GetErrNo();
            if(returnOk == SFS_NO_ERR) 
                while(1);
        }
        OS_TimeDly(1);
    }
//    returnOk = SFS_CloseUser();
}

void Task2(void *Tdata) KC51_REENTRANT
{
    U08            returnOk;
    U08            Readed[20];
    SFS_HANDLE     fhandle;
    SFS_LONG       written, loop = 0;

    Tdata = Tdata;

    returnOk = OS_TimeDlyResume(2);
    returnOk = SFS_BecomeUser(&SFS_T2);
    if(returnOk == SFS_NO_ERR) {
        while(1) {
            fhandle=SFS_OpenFile("/Dir2/Dir21/config", SFS_ACC_RW);
            if(fhandle) {
                returnOk = SFS_SeekFile(45, fhandle);
                if(returnOk == SFS_NO_ERR) {
                    written = SFS_WriteFile(Text , sizeof(Text), fhandle);
                    if(written != sizeof(Text))
                        returnOk = SFS_GetErrNo();
                    if(returnOk == SFS_NO_ERR) {
                        returnOk = SFS_SeekFile(45, fhandle);
                        if(returnOk == SFS_NO_ERR) {
                            written = SFS_ReadFile((U08 OS_HUGE *)Readed, sizeof(Text), fhandle);
                            if(written != sizeof(Text))
                                returnOk = SFS_GetErrNo();
                            else if(0 != SFSstrcmp((SFS_PATHNAME OS_HUGE *)Text, (SFS_PATHNAME OS_HUGE *)Readed))
                                while(1);
                        }
                    }
                }
                returnOk = SFS_CloseFile(fhandle);
                // ---- HW specific ----
                LED_toggle();
                // ---------------------
            } else {
                returnOk = SFS_GetErrNo();
                if(returnOk == SFS_NO_ERR) 
                    while(1);
            }
            loop++;
            OS_TimeDly(1);
        }
//        returnOk = SFS_CloseUser();
    }
}



