/***********************************************************
*                       pC/SFS 1.14b                       *
*            a FREE - small Serial File System             *
*                                                          *
*           by Frank Goetze  -  www.embedded-os.de         *
************************************************************
*                   SFS_Port_SPI_ARC_EM.c                  *
*         Synopsys ARC-EM Starter Kit SPI xRAM PORT        *
************************************************************
* Revision-Info:                                           *
*                                                          *
***********************************************************/

#include "../../../inc/LLdriver_HW/SFS_Port_SPI_HWxxxx.h"

#include "ARC_EM_SPI_regs.h"

/**********************************************************
 *   ARC-EM Starter Kit configuration
 ***********************************************************
 *   Pmod5[7] -------------------------------- /CS     ->  (manually used Port A[24])
 *   Pmod5[2] -------------------------------- MOSI    ->
 *   Pmod5[3] -------------------------------- MISO    <-
 *   Pmod5[4] -------------------------------- SCK     ->
 ***********************************************************
 *   Pmod6[7] -------------------------------- /CS     ->  (manually used Port A[28])
 *   Pmod6[2] -------------------------------- MOSI    ->
 *   Pmod6[3] -------------------------------- MISO    <-
 *   Pmod6[4] -------------------------------- SCK     ->
 **********************************************************/
#define         SPI_PMOD           5

#define         SPIx               0
#if (SPI_PMOD == 5)
 #define        SPI_CS            (1 << 24)
 #define        PMx_UR_SPI_M0     PM5_UR_SPI_M0
#else
 #define        SPI_CS            (1 << 28)
 #define        PMx_UR_SPI_M0     PM6_UR_SPI_M0
#endif
DW_SPI_REG_PTR  spi_reg_ptr     = (DW_SPI_REG_PTR) (0xF0000000U + REL_REGBASE_SPI0);
MUX_REG *       mux_reg_ptr     = (MUX_REG *)      (0xF0000000U + REL_REGBASE_PINMUX);
MST_REG *       spi_mst_reg_ptr = (MST_REG *)      (0xF0000000U + REL_REGBASE_SPI_MST_CS_CTRL);
DW_GPIO_REG_PTR gpio_reg_ptr    = (DW_GPIO_REG_PTR)(0xF0000000U + REL_REGBASE_GPIO0);

/*
************************************************************
*             LOWLEVEL SET /CS TO ACTIVE
************************************************************
*/
void _sfsPort_CS_en(void)
{
#ifdef _pC_OS_
    OS_Lock();
#endif
    gpio_reg_ptr->SWPORTS[0].DR  &= ~SPI_CS;
}

/*
************************************************************
*             LOWLEVEL RESET /CS TO DEACTIVE
************************************************************
*/
void _sfsPort_CS_dis(void)
{
    gpio_reg_ptr->SWPORTS[0].DR  |= SPI_CS;
#ifdef _pC_OS_
    OS_Unlock();
#endif
}

/*
************************************************************
*             LOWLEVEL BYTE_WISE I/O-FUNCTION
************************************************************
*/
U08 _sfsPort_RW(U08 byte)
{
    while (!(spi_reg_ptr->SR & DW_SPI_SR_TFNF)) _NOP();
    spi_reg_ptr->DATAREG = (U32)byte;
    while (!(spi_reg_ptr->SR & DW_SPI_SR_RFNE)) _NOP();
    return (U08)spi_reg_ptr->DATAREG;  
}

/*
************************************************************
*                LOWLEVEL READ STATUS
************************************************************
*/
U08 _sfsPort_readStatus(U08 cmd)
{
    U08 sr;

    _sfsPort_CS_en();
    _sfsPort_RW(cmd);
    sr = _sfsPort_RW(0xFF);
    _sfsPort_CS_dis();
    return sr;
}

/*
************************************************************
*                 INIT SPI-INTERFACE
************************************************************
*/
void _sfsPort_Init(void)
{
    U32 ctrl0_reg = 0; 

    // switch PM6[0] or PM5[0] to SPI master (use CSx manually)
    mux_reg_ptr[PMOD_MUX_CTRL] = 0;
    mux_reg_ptr[SPI_MAP_CTRL]  = 0;
    mux_reg_ptr[UART_MAP_CTRL] = 0xE4;  // default
    mux_reg_ptr[PMOD_MUX_CTRL] = PM1_UR_UART_0 | PMx_UR_SPI_M0;    // UART0 for later

    // init SPI0 master (Pmod5 or Pmod6)
    spi_reg_ptr->SSIENR = DW_SPI_SSI_DISABLE;
    ctrl0_reg = spi_reg_ptr->ICR;
    ctrl0_reg = DW_SPI_CTRLR0_FRF_MOTOROLA | DW_SPI_TMOD_TRANSMIT_RECEIVE | (SPI_CLK_MODE_DEFAULT << DW_SPI_CTRLR0_SC_OFS) | (SPI_DFS_DEFAULT - 1) | DW_SPI_CTRLR0_SLV_OE_ENABLE;
    spi_reg_ptr->CTRLR0 = ctrl0_reg; 
    spi_reg_ptr->CTRLR1 = 0;
    spi_reg_ptr->SER    = 0; 
    spi_reg_ptr->BAUDR  = (CLK_BUS_APB / 6000000U);      // we use 6MHz
    spi_reg_ptr->TXFTLR = 0;
    spi_reg_ptr->RXFTLR = 0; 
    spi_reg_ptr->IMR    = 0;
    spi_reg_ptr->SER    = (1 << SPIx);
    *spi_mst_reg_ptr   &= ~(1 << SPIx);
    spi_reg_ptr->SSIENR = DW_SPI_SSI_ENABLE;

    // manually use of Port A[2x] as CS-line
    gpio_reg_ptr->SWPORTS[0].DDR |= SPI_CS;
    gpio_reg_ptr->SWPORTS[0].DR  |= SPI_CS;

#if 0
    while(1) {
        _sfsPort_CS_en();
        _sfsPort_RW(0x54);
        _sfsPort_CS_dis();
        _NOP();
    }
#endif
}

/************************* END ****************************/
