/***********************************************************
*                       pC/SFS 1.14b                       *
*            a FREE - small Serial File System             *
*                                                          *
*           by Frank Goetze  -  www.embedded-os.de         *
************************************************************
*                SFS_Port_SPI_GD32VF103C.c                 *
*                GD32VF103C SPI xRAM PORT                  *
************************************************************
* Revision-Info:                                           *
*                                                          *
***********************************************************/

#include "../../../inc/LLdriver_HW/SFS_Port_SPI_HWxxxx.h"

#include "gd32vf103_rcu.h"
#include "gd32vf103_spi.h"
#include "gd32vf103_gpio.h"

/**********************************************************
 *   GigaDevice GD32VF103C configuration
 ***********************************************************
 *   PA4 --- SPI0 ---------------------------- /CS     ->  (manual used)
 *   PA5 --- SPI0 ---------------------------- SCK     ->
 *   PA6 --- SPI0 ---------------------------- MISO    <-
 *   PA7 --- SPI0 ---------------------------- MOSI    ->
 **********************************************************/

/*
************************************************************
*             LOWLEVEL SET /CS TO ACTIVE
************************************************************
*/
void _sfsPort_CS_en(void)
{
#ifdef _pC_OS_
    OS_Lock();
#endif
    gpio_bit_reset(GPIOA,GPIO_PIN_4);
}

/*
************************************************************
*             LOWLEVEL RESET /CS TO DEACTIVE
************************************************************
*/
void _sfsPort_CS_dis(void)
{
    gpio_bit_set(GPIOA,GPIO_PIN_4);
#ifdef _pC_OS_
    OS_Unlock();
#endif
}

/*
************************************************************
*             LOWLEVEL BYTE_WISE I/O-FUNCTION
************************************************************
*/
U08 _sfsPort_RW(U08 byte)
{
    spi_i2s_data_transmit(SPI0, byte);
    while (spi_i2s_flag_get(SPI0, SPI_FLAG_TBE) == RESET) _NOP();        // or SPI_FLAG_TRANS
    while (spi_i2s_flag_get(SPI0, SPI_FLAG_RBNE) == RESET) _NOP();
    return spi_i2s_data_receive(SPI0);
}

/*
************************************************************
*                LOWLEVEL READ STATUS
************************************************************
*/
U08 _sfsPort_readStatus(U08 cmd)
{
    U08 sr;

    _sfsPort_CS_en();
    _sfsPort_RW(cmd);
    sr = _sfsPort_RW(0xFF);
    _sfsPort_CS_dis();
    return sr;
}
/*
************************************************************
*                 INIT SPI-INTERFACE
************************************************************
*/
void _sfsPort_Init(void)
{
    spi_parameter_struct  SPI_InitStruct;

    /* GPIOA Periph clock enable */
    rcu_periph_clock_enable(RCU_GPIOA);

    /* Configure PA05, PA06, PA07 pins */
    gpio_init(GPIOA, GPIO_MODE_AF_PP, GPIO_OSPEED_50MHZ, GPIO_PIN_5 | GPIO_PIN_7);
    gpio_init(GPIOA, GPIO_MODE_IN_FLOATING, GPIO_OSPEED_50MHZ, GPIO_PIN_6);

    /* Configure PA04 pin: out pushpull mode */
    gpio_init(GPIOA, GPIO_MODE_OUT_PP, GPIO_OSPEED_50MHZ, GPIO_PIN_4);

    /* Configure SPI0 */
    rcu_periph_clock_enable(RCU_SPI0);

    spi_i2s_deinit(SPI0);
    spi_struct_para_init(&SPI_InitStruct);
    SPI_InitStruct.device_mode           = SPI_MASTER;
    SPI_InitStruct.trans_mode            = SPI_TRANSMODE_FULLDUPLEX;
    SPI_InitStruct.frame_size            = SPI_FRAMESIZE_8BIT;
    SPI_InitStruct.nss                   = SPI_NSS_SOFT;
    SPI_InitStruct.endian                = SPI_ENDIAN_MSB;
    SPI_InitStruct.clock_polarity_phase  = SPI_CK_PL_LOW_PH_1EDGE;
    SPI_InitStruct.prescale              = SPI_PSC_16;
    spi_init(SPI0, &SPI_InitStruct);
    spi_enable(SPI0);
}

/************************* END ****************************/
