/***********************************************************
*                       pC/SFS 1.14b                       *
*            a FREE - small Serial File System             *
*                                                          *
*           by Frank Goetze  -  www.embedded-os.de         *
************************************************************
*                   SFS_Port_I2C_M2351.c                   *
*                    M2351 I2C xRAM PORT                   *
************************************************************
* Revision-Info:                                           *
*                                                          *
***********************************************************/

#include "../../../inc/LLdriver_HW/SFS_Port_I2C_HWxxxx.h"

#include <m2351.h>

/***********************************************************
 *   Nuvoton PFM-M2351 configuration
 ***********************************************************
 *
 *   PG03 --- I2C1 ----------------------- SDA     ->    (NO internal pull-up possible !)
 *   PG02 --- I2C1 ----------------------- SCL     ->    (NO internal pull-up possible !)
 *
 *   !!! external pull-up's are required (2.2k) !!!
 **********************************************************/

#if defined (__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE == 3U)
 #define I2C1_x I2C1
#else
 #define I2C1_x I2C1_NS
#endif

enum {
  I2C_TRANSFER_WRITE = 0,
  I2C_TRANSFER_READ  = 1,
};

/*
************************************************************
*                LOWLEVEL READ BYTES
************************************************************
*/
U08 _sfsPort_readBytes(U32 addr, U08 *data, SFS_LONG len)
{
    SFS_LONG i = 0;
    U08      c, j = 2;
    
    I2C1_x->CTL0 = ((I2C1_x->CTL0 & ~0x3C) | I2C_CTL0_SI_Msk) | I2C_CTL0_STA_Msk;
    while (len > 0) {
        while(!(I2C1_x->CTL0 & I2C_CTL0_SI_Msk)) _NOP();        
        switch(I2C1_x->STATUS0) {
            case 0x08:
                I2C1_x->DAT = I2C_MEM_CC_ADDRESS | I2C_MEM_CS_ADDRESS | I2C_MEM_BLOCK | I2C_TRANSFER_WRITE;
                c = I2C_CTL_SI;
                break;
            case 0x18:
                I2C1_x->DAT = (addr >> (8*(j-1))) & 0xFF; j--;
                break;
            case 0x28:
                if(j > 0) {
                    I2C1_x->DAT = (addr >> (8*(j-1))) & 0xFF; j--;
                } else c = I2C_CTL_STA_SI;
                break;
            case 0x10:
                I2C1_x->DAT = I2C_MEM_CC_ADDRESS | I2C_MEM_CS_ADDRESS | I2C_MEM_BLOCK | I2C_TRANSFER_READ;
                c = I2C_CTL_SI;
                break;
            case 0x40:
                c = I2C_CTL_SI_AA;
                break;
            case 0x50:
                data[i++] = I2C1_x->DAT; 
                if (i < (len-1)) c = I2C_CTL_SI_AA;
                else             c = I2C_CTL_SI;
                break;
            case 0x58:
                data[i++] = I2C1_x->DAT;
                c = I2C_CTL_STO_SI;
                len = 0;
                break;
            case 0x20:
            case 0x30:
            case 0x38:
            case 0x48:
            default:
                I2C1_x->CTL0 = (I2C1_x->CTL0 & ~0x3C) | I2C_CTL_STO_SI;
                return SFS_PORT_ERR;
                break;
        }
        I2C1_x->CTL0 = (I2C1_x->CTL0 & ~0x3C) | c;
    }
    return SFS_NO_ERR;
}

/*
************************************************************
*                LOWLEVEL WRITE BYTES
************************************************************
*/
U08 _sfsPort_writeBytes(U32 addr, U08 *data, SFS_LONG len)
{
    SFS_LONG i = 0;
    U08      c, j = 2;

    I2C1_x->CTL0 = ((I2C1_x->CTL0 & ~0x3C) | I2C_CTL0_SI_Msk) | I2C_CTL0_STA_Msk;
    while (len > 0) {
        while(!(I2C1_x->CTL0 & I2C_CTL0_SI_Msk)) _NOP();        
        switch(I2C1_x->STATUS0) {
            case 0x08:
                I2C1_x->DAT = I2C_MEM_CC_ADDRESS | I2C_MEM_CS_ADDRESS | I2C_MEM_BLOCK | I2C_TRANSFER_WRITE;
                c = I2C_CTL_SI;
                break;
            case 0x18:
                I2C1_x->DAT = (addr >> (8*(j-1))) & 0xFF; j--;
                break;
            case 0x28:
                if(j > 0) {
                    I2C1_x->DAT = (addr >> (8*(j-1))) & 0xFF; j--;
                } else {
                    if(i < len) { 
                        I2C1_x->DAT = data[i++];
                    } else { 
                        len = 0; c = I2C_CTL_STO_SI; 
                    }
                }
                break;
            case 0x20:
            case 0x30:
            case 0x38:
            default:
                I2C1_x->CTL0 = (I2C1_x->CTL0 & ~0x3C) | I2C_CTL_STO_SI;
                return SFS_PORT_ERR;
                break;
        }
        I2C1_x->CTL0 = (I2C1_x->CTL0 & ~0x3C) | c;
    }
    return SFS_NO_ERR;
}

/*
************************************************************
*                LOWLEVEL FILL BYTES
************************************************************
*/
U08 _sfsPort_fillBytes(U32 addr, U08 value, SFS_LONG len)
{
    SFS_LONG i = 0;
    U08      c, j = 2;

    I2C1_x->CTL0 = ((I2C1_x->CTL0 & ~0x3C) | I2C_CTL0_SI_Msk) | I2C_CTL0_STA_Msk;
    while (len > 0) {
        while(!(I2C1_x->CTL0 & I2C_CTL0_SI_Msk)) _NOP();
        switch(I2C1_x->STATUS0) {
            case 0x08:
                I2C1_x->DAT = I2C_MEM_CC_ADDRESS | I2C_MEM_CS_ADDRESS | I2C_MEM_BLOCK | I2C_TRANSFER_WRITE;
                c = I2C_CTL_SI;
                break;
            case 0x18:
                I2C1_x->DAT = (addr >> (8*(j-1))) & 0xFF; j--;
                break;
            case 0x28:
                if(j > 0) {
                    I2C1_x->DAT = (addr >> (8*(j-1))) & 0xFF; j--;
                } else {
                    if(i < len) { 
                        I2C1_x->DAT = value; i++;
                    } else { 
                        len = 0; c = I2C_CTL_STO_SI; 
                    }
                }
                break;
            case 0x20:
            case 0x30:
            case 0x38:
            default:
                I2C1_x->CTL0 = (I2C1_x->CTL0 & ~0x3C) | I2C_CTL_STO_SI;
                return SFS_PORT_ERR;
                break;
        }
        I2C1_x->CTL0 = (I2C1_x->CTL0 & ~0x3C) | c;
    }
    return SFS_NO_ERR;
}

/*
************************************************************
*                LOWLEVEL WAIT READY
************************************************************
*/
void _sfsPort_writeReady(void)
{
    U08 l = 1;
    
    do {
        I2C1_x->CTL0 = ((I2C1_x->CTL0 & ~0x3C) | I2C_CTL0_SI_Msk) | I2C_CTL0_STA_Msk;
        while(!(I2C1_x->CTL0 & I2C_CTL0_SI_Msk)) _NOP();
        if(I2C1_x->STATUS0 == 0x08) {
            I2C1_x->DAT  = I2C_MEM_CC_ADDRESS | I2C_MEM_CS_ADDRESS | I2C_TRANSFER_WRITE;
            I2C1_x->CTL0 = (I2C1_x->CTL0 & ~0x3C) |  I2C_CTL_SI;
            while(!(I2C1_x->CTL0 & I2C_CTL0_SI_Msk)) _NOP();
            if(I2C1_x->STATUS0 == 0x18) l = 0;
        }
        I2C1_x->CTL0 = (I2C1_x->CTL0 & ~0x3C) | I2C_CTL_STO_SI;        
    } while (l != 0);
}

/*
************************************************************
*                 INIT I2C-INTERFACE
************************************************************
*/
void _sfsPort_Init(void)
{
/* following code snipplet must be part of secure_init, if this here is running in None-Secure_State */
#if defined (__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE == 3U)
    SYS->GPG_MFPL &= ~(SYS_GPG_MFPL_PG2MFP_Msk | SYS_GPG_MFPL_PG3MFP_Msk);
    SYS->GPG_MFPL |= (SYS_GPG_MFPL_PG2MFP_I2C1_SCL | SYS_GPG_MFPL_PG3MFP_I2C1_SDA); 
    CLK_EnableModuleClock(I2C1_MODULE);
#endif
    
    I2C1_x->CTL0  &= ~I2C_CTL0_I2CEN_Msk;                  // I2C off
    I2C1_x->CLKDIV = 79;
    I2C1_x->CTL0  |= I2C_CTL0_I2CEN_Msk;
}

/************************* END ****************************/
