/***********************************************************
*                       pC/SFS 1.14b                       *
*            a FREE - small Serial File System             *
*                                                          *
*           by Frank Goetze  -  www.embedded-os.de         *
************************************************************
*                   SFS_Port_SPI_M2351.c                   *
*                    M2351 SPI xRAM PORT                   *
************************************************************
* Revision-Info:                                           *
*                                                          *
***********************************************************/

#include "../../../inc/LLdriver_HW/SFS_Port_SPI_HWxxxx.h"

#include <m2351.h>

/***********************************************************
 *   Nuvoton PFM-M2351 configuration
 ***********************************************************
 *   PF06 --- SPI0 ----------------------- MOSI    ->
 *   PF08 --- SPI0 ----------------------- SCK     ->
 *   PF09 --- SPI0 ----------------------- /CS     ->  (none auto)
 *   PF07 --- SPI0 ----------------------- MISO    <-
 **********************************************************/

#if defined (__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE == 3U)
 #define SPI0_x SPI0
#else
 #define SPI0_x SPI0_NS
#endif

/*
************************************************************
*             LOWLEVEL SET /CS TO ACTIVE
************************************************************
*/
void _sfsPort_CS_en(void)
{
#ifdef _pC_OS_
    OS_Lock();
#endif
    SPI0_x->SSCTL |= SPI_SSCTL_SS_Msk;
}
/*
************************************************************
*             LOWLEVEL RESET /CS TO DEACTIVE
************************************************************
*/
void _sfsPort_CS_dis(void)
{
    SPI0_x->SSCTL &= ~SPI_SSCTL_SS_Msk;
#ifdef _pC_OS_
    OS_Unlock();
#endif
}

/*
************************************************************
*             LOWLEVEL BYTE_WISE I/O-FUNCTION
************************************************************
*/
U08 _sfsPort_RW(U08 byte)
{
    while(!(SPI0_x->STATUS & SPI_STATUS_TXEMPTY_Msk)) __NOP();
    SPI0_x->TX = byte;
    while(SPI0_x->STATUS & SPI_STATUS_RXEMPTY_Msk) __NOP();
    return (U08)SPI0_x->RX;
}

/*
************************************************************
*                LOWLEVEL READ STATUS
************************************************************
*/
U08 _sfsPort_readStatus(U08 cmd)
{
    U08 sr;

    _sfsPort_CS_en();
    _sfsPort_RW(cmd);
    sr = _sfsPort_RW(0xFF);
    _sfsPort_CS_dis();
    return sr;
}
/*
************************************************************
*                 INIT SPI-INTERFACE
************************************************************
*/
void _sfsPort_Init(void)
{
/* following code snipplet must be part of secure_init, if this here is running in None-Secure_State */
#if defined (__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE == 3U)
    SYS->GPF_MFPL = ((SYS->GPF_MFPL & ~(SYS_GPF_MFPL_PF6MFP_Msk)) | SYS_GPF_MFPL_PF6MFP_SPI0_MOSI);
    SYS->GPF_MFPL = ((SYS->GPF_MFPL & ~(SYS_GPF_MFPL_PF7MFP_Msk)) | SYS_GPF_MFPL_PF7MFP_SPI0_MISO);
    SYS->GPF_MFPH = ((SYS->GPF_MFPH & ~(SYS_GPF_MFPH_PF8MFP_Msk)) | SYS_GPF_MFPH_PF8MFP_SPI0_CLK);
    SYS->GPF_MFPH = ((SYS->GPF_MFPH & ~(SYS_GPF_MFPH_PF9MFP_Msk)) | SYS_GPF_MFPH_PF9MFP_SPI0_SS);
    CLK_SetModuleClock(SPI0_MODULE, CLK_CLKSEL2_SPI0SEL_PCLK1, MODULE_NoMsk);
    CLK_EnableModuleClock(SPI0_MODULE);
#endif

    SPI0_x->CTL     = 0;                                   // SPI off
    SPI0_x->I2SCTL &= ~SPI_I2SCTL_I2SEN_Msk;
    SPI0_x->SSCTL   = SPI_SS_ACTIVE_LOW;
    SPI0_x->FIFOCTL = SPI_FIFOCTL_RXFBCLR_Msk | SPI_FIFOCTL_TXFBCLR_Msk | SPI_FIFOCTL_RXRST_Msk | SPI_FIFOCTL_TXRST_Msk;
    while (SPI0_x->STATUS & SPI_STATUS_TXRXRST_Msk) _NOP();
    SPI0_x->CLKDIV  = 1;                                   // clk-source / 2
    SPI0_x->CTL     = SPI_CTL_TXNEG_Msk | (8 << SPI_CTL_DWIDTH_Pos) | SPI_CTL_SPIEN_Msk;
}

/************************* END ****************************/
