/***********************************************************
*                       pC/SFS 1.14b                       *
*            a FREE - small Serial File System             *
*                                                          *
*           by Frank Goetze  -  www.embedded-os.de         *
************************************************************
*                   SFS_Port_I2C_SAML11.c                  *
*                   SAML11 I2C xRAM PORT                   *
************************************************************
* Revision-Info:                                           *
*                                                          *
***********************************************************/

#include "../../../inc/LLdriver_HW/SFS_Port_I2C_HWxxxx.h"

#include <sam.h>

/***********************************************************
 *   Atmel SAML11E16 configuration
 ***********************************************************
 *
 *   PA22 --- SERCOM0.0 ----------------------- SDA     ->    (NO internal pull-up possible !)
 *   PA23 --- SERCOM0.1 ----------------------- SCL     ->    (NO internal pull-up possible !)
 *
 *   !!! external pull-up's are required (2.2k) !!!
 **********************************************************/

enum {
  I2C_TRANSFER_WRITE = 0,
  I2C_TRANSFER_READ  = 1,
};

/*
************************************************************
*                LOWLEVEL READ BYTES
************************************************************
*/
U08 _sfsPort_readBytes(U32 addr, U08 *data, SFS_LONG len)
{
    SFS_LONG i;

    SERCOM0->I2CM.ADDR.reg = I2C_MEM_CC_ADDRESS | I2C_MEM_CS_ADDRESS | I2C_MEM_BLOCK | I2C_TRANSFER_WRITE;
    while (0 == (SERCOM0->I2CM.INTFLAG.reg & SERCOM_I2CM_INTFLAG_MB)) _NOP();
    if (SERCOM0->I2CM.STATUS.reg & SERCOM_I2CM_STATUS_RXNACK) {
        SERCOM0->I2CM.CTRLB.reg |= SERCOM_I2CM_CTRLB_CMD(3);
        return SFS_PORT_ERR;
    }
    for (i = 2; i > 0; i--) {
        SERCOM0->I2CM.DATA.reg = (addr >> (8*(i-1))) & 0xFF;
        while (0 == (SERCOM0->I2CM.INTFLAG.reg & SERCOM_I2CM_INTFLAG_MB)) _NOP();
        if (SERCOM0->I2CM.STATUS.reg & SERCOM_I2CM_STATUS_RXNACK) {
            SERCOM0->I2CM.CTRLB.reg |= SERCOM_I2CM_CTRLB_CMD(3);
            return SFS_PORT_ERR;
        }
    }
    SERCOM0->I2CM.ADDR.reg = I2C_MEM_CC_ADDRESS | I2C_MEM_CS_ADDRESS | I2C_MEM_BLOCK | I2C_TRANSFER_READ;
    while (0 == (SERCOM0->I2CM.INTFLAG.reg & SERCOM_I2CM_INTFLAG_SB)) _NOP();
    if (SERCOM0->I2CM.STATUS.reg & SERCOM_I2CM_STATUS_RXNACK) {
        SERCOM0->I2CM.CTRLB.reg |= SERCOM_I2CM_CTRLB_CMD(3);
        return SFS_PORT_ERR;
    }
    SERCOM0->I2CM.CTRLB.reg &= ~SERCOM_I2CM_CTRLB_ACKACT;
    for(i = 0; i < (len-1); i++) {
        data[i] = SERCOM0->I2CM.DATA.reg;
        while (0 == (SERCOM0->I2CM.INTFLAG.reg & SERCOM_I2CM_INTFLAG_SB)) _NOP();
    }
    SERCOM0->I2CM.CTRLB.reg |= SERCOM_I2CM_CTRLB_ACKACT;
    if(len) {
        SERCOM0->I2CM.CTRLB.reg |= SERCOM_I2CM_CTRLB_CMD(3);
        data[len-1] = SERCOM0->I2CM.DATA.reg;
    }
    return SFS_NO_ERR;
}

/*
************************************************************
*                LOWLEVEL WRITE BYTES
************************************************************
*/
U08 _sfsPort_writeBytes(U32 addr, U08 *data, SFS_LONG len)
{
    SFS_LONG i;

    SERCOM0->I2CM.ADDR.reg = I2C_MEM_CC_ADDRESS | I2C_MEM_CS_ADDRESS | I2C_MEM_BLOCK | I2C_TRANSFER_WRITE;
    while (0 == (SERCOM0->I2CM.INTFLAG.reg & SERCOM_I2CM_INTFLAG_MB)) _NOP();
    if (SERCOM0->I2CM.STATUS.reg & SERCOM_I2CM_STATUS_RXNACK) {
        SERCOM0->I2CM.CTRLB.reg |= SERCOM_I2CM_CTRLB_CMD(3);
        return SFS_PORT_ERR;
    }
    for (i = 2; i > 0; i--) {
        SERCOM0->I2CM.DATA.reg = (addr >> (8*(i-1))) & 0xFF;
        while (0 == (SERCOM0->I2CM.INTFLAG.reg & SERCOM_I2CM_INTFLAG_MB)) _NOP();
        if (SERCOM0->I2CM.STATUS.reg & SERCOM_I2CM_STATUS_RXNACK) {
            SERCOM0->I2CM.CTRLB.reg |= SERCOM_I2CM_CTRLB_CMD(3);
            return SFS_PORT_ERR;
        }
    }
    for (i = 0; i < len; i++) {
        SERCOM0->I2CM.DATA.reg = data[i];
        while (0 == (SERCOM0->I2CM.INTFLAG.reg & SERCOM_I2CM_INTFLAG_MB)) _NOP();
        if (SERCOM0->I2CM.STATUS.reg & SERCOM_I2CM_STATUS_RXNACK) {
            SERCOM0->I2CM.CTRLB.reg |= SERCOM_I2CM_CTRLB_CMD(3);
            return SFS_PORT_ERR;
        }
    }
    SERCOM0->I2CM.CTRLB.reg |= SERCOM_I2CM_CTRLB_CMD(3);
    return SFS_NO_ERR;
}

/*
************************************************************
*                LOWLEVEL FILL BYTES
************************************************************
*/
U08 _sfsPort_fillBytes(U32 addr, U08 value, SFS_LONG len)
{
    SFS_LONG i;

    SERCOM0->I2CM.ADDR.reg = I2C_MEM_CC_ADDRESS | I2C_MEM_CS_ADDRESS | I2C_MEM_BLOCK | I2C_TRANSFER_WRITE;
    while (0 == (SERCOM0->I2CM.INTFLAG.reg & SERCOM_I2CM_INTFLAG_MB)) _NOP();
    if (SERCOM0->I2CM.STATUS.reg & SERCOM_I2CM_STATUS_RXNACK) {
        SERCOM0->I2CM.CTRLB.reg |= SERCOM_I2CM_CTRLB_CMD(3);
        return SFS_PORT_ERR;
    }
    for (i = 2; i > 0; i--) {
        SERCOM0->I2CM.DATA.reg = (addr >> (8*(i-1))) & 0xFF;
        while (0 == (SERCOM0->I2CM.INTFLAG.reg & SERCOM_I2CM_INTFLAG_MB)) _NOP();
        if (SERCOM0->I2CM.STATUS.reg & SERCOM_I2CM_STATUS_RXNACK) {
            SERCOM0->I2CM.CTRLB.reg |= SERCOM_I2CM_CTRLB_CMD(3);
            return SFS_PORT_ERR;
        }
    }
    for (i = 0; i < len; i++) {
        SERCOM0->I2CM.DATA.reg = value;
        while (0 == (SERCOM0->I2CM.INTFLAG.reg & SERCOM_I2CM_INTFLAG_MB)) _NOP();
        if (SERCOM0->I2CM.STATUS.reg & SERCOM_I2CM_STATUS_RXNACK) {
            SERCOM0->I2CM.CTRLB.reg |= SERCOM_I2CM_CTRLB_CMD(3);
            return SFS_PORT_ERR;
        }
    }
    SERCOM0->I2CM.CTRLB.reg |= SERCOM_I2CM_CTRLB_CMD(3);
    return SFS_NO_ERR;
}

/*
************************************************************
*                LOWLEVEL WAIT READY
************************************************************
*/
void _sfsPort_writeReady(void)
{
    U32 addr = 0;

    do {
        SERCOM0->I2CM.ADDR.reg = I2C_MEM_CC_ADDRESS | I2C_MEM_CS_ADDRESS | I2C_MEM_BLOCK | I2C_TRANSFER_WRITE;
        while (0 == (SERCOM0->I2CM.INTFLAG.reg & SERCOM_I2CM_INTFLAG_MB)) _NOP();
        SERCOM0->I2CM.CTRLB.reg |= SERCOM_I2CM_CTRLB_CMD(3);
    }
    while (SERCOM0->I2CM.STATUS.reg & SERCOM_I2CM_STATUS_RXNACK);
}

/*
************************************************************
*                 INIT I2C-INTERFACE
************************************************************
*/
void _sfsPort_Init(void)
{
/* following code snipplet must be part of secure_init, if this here is running in None-Secure_State */
#if defined (__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE == 3U)
    MCLK->APBCMASK.reg |= MCLK_APBCMASK_SERCOM0;
    GCLK->PCHCTRL[SERCOM0_GCLK_ID_CORE].reg = GCLK_PCHCTRL_GEN(0) | GCLK_PCHCTRL_CHEN;
    while(!(GCLK->PCHCTRL[SERCOM0_GCLK_ID_CORE].bit.CHEN));
#endif
    SERCOM0->I2CM.CTRLA.reg = SERCOM_I2CM_CTRLA_SWRST;
    while(SERCOM0->I2CM.SYNCBUSY.bit.SWRST) __NOP();
    SERCOM0->I2CM.CTRLA.reg = SERCOM_I2CM_CTRLA_SPEED(1) | SERCOM_I2CM_CTRLA_SDAHOLD(0x2) | 
                              SERCOM_I2CM_CTRLA_RUNSTDBY | SERCOM_I2CM_CTRLA_MODE(5);
    SERCOM0->I2CM.CTRLB.reg = SERCOM_I2CM_CTRLB_SMEN | SERCOM_I2CM_CTRLB_ACKACT;
    while(SERCOM0->I2CM.SYNCBUSY.bit.SYSOP) __NOP();
    SERCOM0->I2CM.BAUD.reg = SERCOM_I2CM_BAUD_BAUD(11) | SERCOM_I2CM_BAUD_BAUDLOW(22);
    while(SERCOM0->I2CM.SYNCBUSY.bit.SYSOP) __NOP();
    PORT->Group[0].PINCFG[PIN_PA22].bit.PMUXEN = 1;
    PORT->Group[0].PINCFG[PIN_PA23].bit.PMUXEN = 1;
    PORT->Group[0].PMUX[PIN_PA22/2].reg = (PORT_PMUX_PMUXE(2) | PORT_PMUX_PMUXO(2));
    SERCOM0->I2CM.CTRLA.reg |= SERCOM_I2CM_CTRLA_ENABLE;
    while(SERCOM0->I2CM.SYNCBUSY.bit.ENABLE) __NOP();
    while(!(SERCOM0->I2CM.STATUS.reg & SERCOM_I2CM_STATUS_BUSSTATE(1))) {
        SERCOM0->I2CM.STATUS.reg = SERCOM_I2CM_STATUS_BUSSTATE(1); 
    }
}

/************************* END ****************************/
